/*
    Copyright (C) 2008-2017, Millistream Market Data <support@millistream.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MDFCRYPT_H
# define MDFCRYPT_H 1

#define MIN_DH_LENGTH 256
#define MAX_DH_LENGTH (MIN_DH_LENGTH * 4)

#define MAX_ENCRYPTED_MESSAGES_PER_KEY 4294967295

# if defined WHIRLPOOL_DIGEST_LENGTH && !defined OPENSSL_NO_WHIRLPOOL
#  define MAX_DIGEST_LENGTH WHIRLPOOL_DIGEST_LENGTH
# elif defined SHA512_DIGEST_LENGTH && !defined OPENSSL_NO_SHA512
#  define MAX_DIGEST_LENGTH SHA512_DIGEST_LENGTH
# elif defined SHA256_DIGEST_LENGTH && !defined OPENSSL_NO_SHA256
#  define MAX_DIGEST_LENGTH SHA256_DIGEST_LENGTH
# elif defined SHA_DIGEST_LENGTH && !defined OPENSSL_NO_SHA1
#  define MAX_DIGEST_LENGTH SHA_DIGEST_LENGTH
# elif defined RIPEMD160_DIGEST_LENGTH && !defined OPENSSL_NO_RIPEMD
#  define MAX_DIGEST_LENGTH RIPEMD160_DIGEST_LENGTH
# elif defined MD5_DIGEST_LENGTH && !defined OPENSSL_NO_MD5
#  define MAX_DIGEST_LENGTH MD5_DIGEST_LENGTH
# else
#  error No supported Digest Method avilable!
# endif

struct cryptosystem {
	int type;
	int size;
	char *name;
};

union digest_ctx {
#if defined MD5_DIGEST_LENGTH && !defined OPENSSL_NO_MD5
	MD5_CTX md5;
#endif
#if defined SHA_DIGEST_LENGTH && !defined OPENSSL_NO_SHA1
	SHA_CTX sha1;
#endif
#if defined RIPEMD160_DIGEST_LENGTH && !defined OPENSSL_NO_RIPEMD
	RIPEMD160_CTX ripemd160;
#endif
#if defined SHA256_DIGEST_LENGTH && !defined OPENSSL_NO_SHA256
	SHA256_CTX sha256;
#endif
#if defined SHA512_DIGEST_LENGTH && !defined OPENSSL_NO_SHA512
	SHA512_CTX sha512;
#endif
#if defined WHIRLPOOL_DIGEST_LENGTH && !defined OPENSSL_NO_WHIRLPOOL
	WHIRLPOOL_CTX whirlpool;
#endif
};

RSA *mdf_int_load_rsapublickey ();
int mdf_int_sizeof_cryptmethods ();
void mdf_int_populate_cryptmethods (uint8_t *message, int *position);
int mdf_int_digests_verify (const int type, int * restrict method, int * restrict length);
int mdf_int_encryptions_verify (const int type, int *method);
int mdf_int_digest_init (const int digest_method, union digest_ctx *ctx);
int mdf_int_digest_update (const int digest_method, union digest_ctx *ctx, const void *data, size_t len);
int mdf_int_digest_final (const int digest_method, unsigned char *md, union digest_ctx *ctx);
int mdf_int_digest_verify (const int digest_method, unsigned char * restrict md, unsigned char * restrict sig, unsigned int siglen, RSA *rsa);
void mdf_int_set_keys (mdf_t handle, unsigned char * restrict nonce, int nonce_len, AES_KEY **key, HMAC_CTX **hmac, unsigned char ** restrict iv);
void mdf_int_handle_server_rekey (mdf_t handle, uint8_t *data, const size_t tlen);

#endif

