/*
    Copyright (C) 2008-2015, Millistream Market Data <support@millistream.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef _MDF_BYTESWAP_H
#define _MDF_BYTESWAP_H 1

#ifdef HAVE_ENDIAN_H
# include <endian.h>
#else
# define __LITTLE_ENDIAN 1234
# define __BIG_ENDIAN 4321
# define __BYTE_ORDER __LITTLE_ENDIAN
#endif

#ifdef HAVE_BYTESWAP_H
# include <byteswap.h>
#else
# define bswap_16(x) \
({ \
	uint16_t __x = (x); \
	((uint16_t)( \
		(((uint16_t)(__x) & (uint16_t)0x00ffU) << 8) | \
		(((uint16_t)(__x) & (uint16_t)0xff00U) >> 8) )); \
})

# define bswap_32(x) \
({ \
	uint32_t __x = (x); \
	((uint32_t)( \
		(((uint32_t)(__x) & (uint32_t)0x000000ffUL) << 24) | \
		(((uint32_t)(__x) & (uint32_t)0x0000ff00UL) <<  8) | \
		(((uint32_t)(__x) & (uint32_t)0x00ff0000UL) >>  8) | \
		(((uint32_t)(__x) & (uint32_t)0xff000000UL) >> 24) )); \
})

# define bswap_64(x) \
({ \
	uint64_t __x = (x); \
	((uint64_t)( \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00000000000000ffULL) << 56) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x000000000000ff00ULL) << 40) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x0000000000ff0000ULL) << 24) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00000000ff000000ULL) <<  8) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x000000ff00000000ULL) >>  8) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x0000ff0000000000ULL) >> 24) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00ff000000000000ULL) >> 40) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0xff00000000000000ULL) >> 56) )); \
})
#endif

#if __BYTE_ORDER == __BIG_ENDIAN
# define cpu_to_le16(v) bswap_16(v)
# define cpu_to_le32(v) bswap_32(v)
# define cpu_to_le64(v) bswap_64(v)
# define cpu_to_be64(v) (v)
# define le16_to_cpu(v) bswap_16(v)
# define le32_to_cpu(v) bswap_32(v)
# define le64_to_cpu(v) bswap_64(v)
# define be64_to_cpu(v) (v)
#else
# define cpu_to_le16(v) (v)
# define cpu_to_le32(v) (v)
# define cpu_to_le64(v) (v)
# define cpu_to_be64(v) bswap_64(v)
# define le16_to_cpu(v) (v)
# define le32_to_cpu(v) (v)
# define le64_to_cpu(v) (v)
# define be64_to_cpu(v) bswap_64(v)
#endif

#endif

